#ifndef MHEXABRICKSTRAINMATRIXIMPL_H_
#define MHEXABRICKSTRAINMATRIXIMPL_H_

#include <memory>
#include <org.sipesc.core.engdbs.data.mdataobject.h>
#include <org.sipesc.core.engdbs.data.mdatamodel.h>
#include <org.sipesc.core.utility.progressmonitor.h>
#include <org.sipesc.fems.matrix.mmatrixfactory.h>
#include <org.sipesc.fems.femsutils.mintegpoint.h>
#include <org.sipesc.fems.hexaelement.mhexabrickstrainmatrix.h>
using namespace org::sipesc::core::engdbs::data;
using namespace org::sipesc::core::utility;
using namespace org::sipesc::fems::matrix;
using namespace org::sipesc::fems::femsutils;
using namespace org::sipesc::fems::hexaelement;

class MHexaBrickStrainMatrixImpl: public MHexaBrickStrainMatrixInterface
  {
  public:
    MHexaBrickStrainMatrixImpl();
    /**
     * 析构函数. 仅为类继承关系而设置.
     */
    virtual ~MHexaBrickStrainMatrixImpl();
    /**
     * initialize Task object with given parent object, provider and receiver
     *
     * @param parentTask parent task
     * @param model object of MDataModel
     * @return if succeed, return true, otherwise return false
     */
    bool initialize(MDataModel& model, bool isRepeated);
    /**
     * 取进度监视对象
     */
    ProgressMonitor getProgressMonitor() const;
    /**
     * to set element node coordinate
     *
     * @param eleData data which stores element node information
     */
    void setState(const MDataObject& localData);
    /**
     * to get strain matrix
     *
     * @param integ data which stores parameter coordinate of integral point
     * @return strain matrix
     */
    MMatrix getStrainMatrix(const MIntegPoint& integ);
    /**
     * to get jacobi determinant value
     *
     * @return jacobi determinant value
     */
    double getJacobiValue() const;
    /**
     * to transfer local to global coordinate
     *
     * @param integ data which stores parameter coordinate of integral point
     * @return global coordinate
     */
    QList<double> coordTrans(const MIntegPoint& integ) const;

  private:
    class Data;
    std::auto_ptr<Data> _data;
  };

#endif /* MHEXABRICKSTRAINMATRIXIMPL_H_ */
