#include <mbrickelestiffbuilderimpl.h>
#include <mextensionmanager.h>
#include <mobjectmanager.h>
#include <mbrickelestiffcalculatorimpl.h>
#include <org.sipesc.core.utility.utilitymanager.h>
#include <org.sipesc.utilities.mextensionfactorymanager.h>
#include <org.sipesc.fems.element.melementstrainmatrix.h>
#include <org.sipesc.fems.element.mintegform.h>


#include <qdebug.h>
using namespace org::sipesc::utilities;

class MBrickEleStiffBuilderImpl::Data
  {
  public:
    Data()
      {
      _isInitialized = false;
      }

  public:
    MExtensionManager _extManager;
    ProgressMonitor _monitor;
    MElementStrainMatrix _B;
    MIntegForm _integ;
    MExtensionFactoryManager _factoryManager;
    MBrickEleStiffCalculatorImpl* _p;
    bool _isInitialized;
  };

MBrickEleStiffBuilderImpl::MBrickEleStiffBuilderImpl()
  {
  _data.reset(new MBrickEleStiffBuilderImpl::Data);

  if (!_data.get())
    mReportError(M_ERROR_FATAL, "MBrickEleStiffBuilderImpl::"
        "MBrickEleStiffBuilderImpl() ****** failed");

  _data->_p = new MBrickEleStiffCalculatorImpl;
  Q_ASSERT(_data->_p);
  _data->_extManager = MExtensionManager::getManager();

  MObjectManager objManager = MObjectManager::getManager();
  UtilityManager util = objManager.getObject(
      "org.sipesc.core.utility.utilitymanager");
  Q_ASSERT(!util.isNull());
  _data->_monitor = util.createProgressMonitor("MBrickEleStiffBuilder",
      ProgressMonitor());
  Q_ASSERT(!_data->_monitor.isNull());
  }

MBrickEleStiffBuilderImpl::~MBrickEleStiffBuilderImpl()
  {
  }

void MBrickEleStiffBuilderImpl::setStrainType(const QString& strainMatrix)
  {
  Q_ASSERT(!_data->_isInitialized);
  //通过插件找工厂管理器

  MExtensionFactoryManager factoryManager =
      _data->_extManager.createExtension(
          "org.sipesc.utilities.MExtensionFactoryManager");

  bool isOk = factoryManager.initialize("fems.factory.strainmatrix");
  Q_ASSERT(isOk);
  //应变矩阵计算类
  MExtensionFactory strainMatrixFactory = factoryManager.getFactory(
      strainMatrix);
  Q_ASSERT(!strainMatrixFactory.isNull());
  _data->_B = strainMatrixFactory.createExtension();
  Q_ASSERT(!_data->_B.isNull());
  }

void MBrickEleStiffBuilderImpl::setIntegType(const QString& integForm)
  {
  Q_ASSERT(!_data->_isInitialized);
  //通过插件找工厂管理器
  MExtensionFactoryManager factoryManager =
      _data->_extManager.createExtension(
          "org.sipesc.utilities.MExtensionFactoryManager");

  bool isOk = factoryManager.initialize("fems.factory.integrationform");
  Q_ASSERT(isOk);
  //积分形式解析类
  MExtensionFactory integFactory = factoryManager.getFactory(integForm);
  Q_ASSERT(!integFactory.isNull());
  _data->_integ = integFactory.createExtension();
  Q_ASSERT(!_data->_integ.isNull());
  }

void MBrickEleStiffBuilderImpl::setConstType(const QString& eleConst)
  {
  Q_ASSERT(!_data->_isInitialized);
  //通过插件找工厂管理器
  _data->_factoryManager = _data->_extManager.createExtension(
      "org.sipesc.utilities.MExtensionFactoryManager");

  bool isOk = _data->_factoryManager.initialize("fems.factory." + eleConst);
  Q_ASSERT(isOk);
  }

bool MBrickEleStiffBuilderImpl::initialize(MDataModel& model,
    bool isRepeated)
  {
  if (_data->_isInitialized)
    return false; //不能重复初始化

  bool isOk = _data->_p->initialize(model, isRepeated);
  Q_ASSERT(isOk);

  _data->_isInitialized = true;
  return true;
  }

ProgressMonitor MBrickEleStiffBuilderImpl::getProgressMonitor() const
  {
  return _data->_monitor;
  }

MExtension MBrickEleStiffBuilderImpl::build(const QVariant& type)
  {
  Q_ASSERT(_data->_isInitialized);

  _data->_p->setStrainMatrix(_data->_B);
  _data->_p->setIntegForm(_data->_integ);
  _data->_p->setEleConst(_data->_factoryManager);

  return MBrickEleStiffCalculator(_data->_p);
  }
