#include <mbrickelestiffcalculatorimpl.h>
#include <qdebug.h>
#include <mextensionmanager.h>
#include <mobjectmanager.h>
#include <org.sipesc.core.utility.utilitymanager.h>
#include <org.sipesc.core.engdbs.data.mdatabasemanager.h>
#include <org.sipesc.fems.global.mfemsglobal.h>
#include <org.sipesc.fems.global.melementsglobal.h>
#include <org.sipesc.utilities.mextensionfactorymanager.h>
#include <org.sipesc.fems.matrix.mmatrixfactory.h>
#include <org.sipesc.fems.matrix.melementstiffmatrixdata.h>
#include <org.sipesc.fems.data.mpropertyrefdata.h>
#include <org.sipesc.fems.data.mpropertydata.h>
#include <org.sipesc.fems.data.melementdata.h>
#include <org.sipesc.fems.data.melementlocalcoorddata.h>
#include <org.sipesc.fems.element.meleconstidparser.h>
#include <org.sipesc.fems.element.mbdbopera.h>
#include <org.sipesc.fems.femsutils.mintegpoint.h>
using namespace org::sipesc::fems::global;
using namespace org::sipesc::utilities;
using namespace org::sipesc::fems::matrix;
using namespace org::sipesc::fems::data;
using namespace org::sipesc::fems::femsutils;

class MBrickEleStiffCalculatorImpl::Data
  {
  public:
    Data()
      {
      _isInitialized = false;
      }

  public:
    MDataModel _model, _elementLocalCoord;
    MDataManager _propertys, _materials, _coordTransMatrixs;
    MObjectManager _objManager;
    MDatabaseManager _baseManager;
    ProgressMonitor _monitor;
    MElementStrainMatrix _B;
    MIntegForm _integ;
    MExtensionFactoryManager _eleConstiManager;
    MMatrixFactory _mFactory;
    QMap<QString, MEleConstiDParser> _eleConstiDParserMap;
    QString _mId;
    bool _isInitialized;

  public:
    /*
     * 查找单元本构解析对象
     */
    MEleConstiDParser findEleConstiDParser(const QString& matType)
      {
      //首先在Map中查找
      if (_eleConstiDParserMap.contains(matType))
        return _eleConstiDParserMap[matType];

      MExtensionFactory eleConstiFactory = _eleConstiManager.getFactory(
          matType);
      Q_ASSERT(!eleConstiFactory.isNull());
      MEleConstiDParser eleConstiDParser =
          eleConstiFactory.createExtension();
      Q_ASSERT(!eleConstiDParser.isNull());

      if (eleConstiDParser.initialize(_model))
        _eleConstiDParserMap.insert(matType, eleConstiDParser);

      return eleConstiDParser;
      }
  };

MBrickEleStiffCalculatorImpl::MBrickEleStiffCalculatorImpl()
  {
  _data.reset(new MBrickEleStiffCalculatorImpl::Data);

  if (!_data.get())
    mReportError(M_ERROR_FATAL, "MBrickEleStiffCalculatorImpl::"
        "MBrickEleStiffCalculatorImpl() ****** failed");

  _data->_objManager = MObjectManager::getManager();
  UtilityManager util = _data->_objManager.getObject(
      "org.sipesc.core.utility.utilitymanager");
  Q_ASSERT(!util.isNull());
  _data->_monitor = util.createProgressMonitor("MBrickEleStiffCalculator",
      ProgressMonitor());
  Q_ASSERT(!_data->_monitor.isNull());
  }
MBrickEleStiffCalculatorImpl::~MBrickEleStiffCalculatorImpl()
  {

  }

void MBrickEleStiffCalculatorImpl::setStrainMatrix(
    const MElementStrainMatrix& B)
  {
  _data->_B = B;
  }

void MBrickEleStiffCalculatorImpl::setIntegForm(const MIntegForm& integ)
  {
  _data->_integ = integ;
  }

void MBrickEleStiffCalculatorImpl::setEleConst(
    const MExtensionFactoryManager& eleConstiManager)
  {
  _data->_eleConstiManager = eleConstiManager;
  }

bool MBrickEleStiffCalculatorImpl::initialize(MDataModel& model,
    bool isRepeated)
  {
  if (_data->_isInitialized)
    return false; //不能重复初始化

  _data->_model = model;
  MFemsGlobal global = _data->_objManager.getObject(
      "org.sipesc.fems.global.femsglobal");
  MElementsGlobal eleGlobal = _data->_objManager.getObject(
      "org.sipesc.fems.global.elementsglobal");
  _data->_mId = eleGlobal.getValue(MElementsGlobal::MaterialId);
  _data->_mFactory = _data->_objManager.getObject(
      "org.sipesc.fems.matrix.matrixfactory");
  Q_ASSERT(!_data->_mFactory.isNull());

  _data->_baseManager = _data->_objManager.getObject(
      "org.sipesc.core.engdbs.mdatabasemanager");
  Q_ASSERT(!_data->_baseManager.isNull());

  //可写方式打开单元坐标转换阵存贮路径
  _data->_elementLocalCoord = _data->_baseManager.createDataModel();
  bool ok = _data->_elementLocalCoord.open(_data->_model,
      global.getValue(MFemsGlobal::ElementLocalCoordPath));
  if (!ok)
    {
    QString errorMessage = "can't open element CoordTransMatrixs DataModel "
        " in MBrickEleStiffCalculatorImpl::initialize() ";
    qDebug() << errorMessage;
    _data->_monitor.setMessage(errorMessage);
    return false;
    }

  MDataModel propertysRef = _data->_baseManager.createDataModel();
  ok = propertysRef.open(_data->_model,
      global.getValue(MFemsGlobal::PropertyRefPath), true);
  if (!ok)
    {
    QString errorMessage = "can't open property DataModel "
        " in MBrickEleStiffCalculatorImpl::initialize() ";
    qDebug() << errorMessage;
    _data->_monitor.setMessage(errorMessage);
    return false;
    }

  _data->_propertys = _data->_baseManager.createDataManager();
  ok = _data->_propertys.open(propertysRef,
      global.getValue(MFemsGlobal::GeneralRef), true);
  if (!ok)
    {
    QString errorMessage = "can't open property DataManager "
        " in MBrickEleStiffCalculatorImpl::initialize() ";
    qDebug() << errorMessage;
    _data->_monitor.setMessage(errorMessage);
    return false;
    }
  //只读方式打开material数据
  _data->_materials = _data->_baseManager.createDataManager();
  ok = _data->_materials.open(_data->_model,
      global.getValue(MFemsGlobal::Material), true);
  if (!ok)
    {
    QString errorMessage = "can't open material DataManager "
        " in MBrickEleStiffCalculatorImpl::initialize() ";
    qDebug() << errorMessage;
    _data->_monitor.setMessage(errorMessage);
    return false;
    }

  //通过插件找工厂管理器
  MExtensionManager extManager = MExtensionManager::getManager();
  MExtensionFactoryManager factoryManager = extManager.createExtension(
      "org.sipesc.utilities.MExtensionFactoryManager");
  bool isOk = factoryManager.initialize("fems.factory.bdboperation");
  Q_ASSERT(isOk);

  _data->_isInitialized = true;
  return true;
  }

ProgressMonitor MBrickEleStiffCalculatorImpl::getProgressMonitor() const
  {
  return _data->_monitor;
  }

MDataObject MBrickEleStiffCalculatorImpl::start(const MDataObject& data)
  {
  Q_ASSERT(_data->_isInitialized);
  MElementData eleData = data;
  Q_ASSERT(!eleData.isNull());

  if (!_data->_coordTransMatrixs.isOpen())
    {
    _data->_coordTransMatrixs = _data->_baseManager.createDataManager();
    QString eleType = eleData.getType();
    bool ok = _data->_coordTransMatrixs.open(_data->_elementLocalCoord,
        eleType, true);
    if (!ok)
      {
      QString errorMessage =
          "can't open element CoordTransMatrixs DataManager "
              " in MBrickEleStiffCalculatorImpl() ";
      qDebug() << errorMessage;
      _data->_monitor.setMessage(errorMessage);
      }
    }

  MElementLocalCoordData eleLocalCoordData;
  eleLocalCoordData = _data->_coordTransMatrixs.getData(eleData.getId());
  _data->_B.setState(eleLocalCoordData);


  MPropertyRefData propertyData;
  propertyData = _data->_propertys.getData(eleData.getPropertyId());
  int matId = propertyData.getPropertyId(_data->_mId);
  MPropertyData matData;
  matData = _data->_materials.getData(matId);
  QString matType = matData.getType();
  MEleConstiDParser eleConstiDParser = _data->findEleConstiDParser(matType);
  eleConstiDParser.setState(eleData);

  //store DNDx ,D  matrix ...at a gaussian point
  MMatrix DNDx, DD;
  double J, wgt;

  bool isConst = eleConstiDParser.isConst();
  if (isConst)
    DD = eleConstiDParser.getConstiDMatrix();

  //  node numbers in a element
  //attention: "gauspointnum != nodeNum";
  int nodeNum=eleData.getNodeCount() ;
  //dim of the element stiffness matrix
  int eleDof = nodeNum* eleData.getActiveDofs(0).count();
  //  store element stiffness matrix
   MMatrix mdn= _data->_mFactory.createMatrix(eleDof, eleDof);

   //qDebug()<<"new method  to calculate the BTDB";
   //numbers of gaussian point in a element
   int count = _data->_integ.getCount();
  for (int i = 0; i < count; i++)
    {
    //qDebug()<<"gauss point num "<<i+1;
    wgt = _data->_integ.getIntegPoint(i).getWeight();
    MIntegPoint integPoint = _data->_integ.getIntegPoint(i);
    //[DNDx]3*8
    DNDx = _data->_B.getStrainMatrix(integPoint);
    J = _data->_B.getJacobiValue();
    //    qDebug()<<"jacobi  = "<<jacobi ;
    if (!isConst)
      {
      QList<double> paraCoord;
      paraCoord = _data->_B.coordTrans(integPoint);
      integPoint.setGlobalCoord(paraCoord);
      DD = eleConstiDParser.getConstiDMatrix(integPoint);
      }

    double* DN_ptr = DNDx.getData();
    double* DD_ptr = DD.getData();
    //store element stiffness matrix of one gaussian point's contribution
    //ATTENTION:重构时候的时候，不要将tmpMatrix 的定义放到积分点循环的外面。会造成计算错误。
    //由于涉及指针操作，挪动这些指针指向的数组的定义位置，需要进行详细的测试。
    MMatrix tmpMatrix = _data->_mFactory.createMatrix(eleDof, eleDof);
    double* tmpMatrix_ptr=tmpMatrix.getData();
    // lower triangular matrix
    //nodeNum=8 or 20.suit for two kinds of elements
    for (int n = 0; n < nodeNum; n++){
      for (int m = n; m < nodeNum; m++){
        //[Ke]3*3 store a block of stiffness matrix
        double Ke[9];  double t[9];
        int posm = m * 3;
        int posn = n * 3;

        t[0] = DN_ptr[m * 3 + 0] * DN_ptr[n * 3 + 0] * J * wgt;
        t[1] = DN_ptr[m * 3 + 0] * DN_ptr[n * 3 + 1] * J * wgt;
        t[2] = DN_ptr[m * 3 + 0] * DN_ptr[n * 3 + 2] * J * wgt;
        t[3] = DN_ptr[m * 3 + 1] * DN_ptr[n * 3 + 0] * J * wgt;
        t[4] = DN_ptr[m * 3 + 1] * DN_ptr[n * 3 + 1] * J * wgt;
        t[5] = DN_ptr[m * 3 + 1] * DN_ptr[n * 3 + 2] * J * wgt;
        t[6] = DN_ptr[m * 3 + 2] * DN_ptr[n * 3 + 0] * J * wgt;
        t[7] = DN_ptr[m * 3 + 2] * DN_ptr[n * 3 + 1] * J * wgt;
        t[8] = DN_ptr[m * 3 + 2] * DN_ptr[n * 3 + 2] * J * wgt;

        Ke[0] = t[0] * DD_ptr[0]  + t[1] * DD_ptr[18] + t[2] * DD_ptr[30]
                  + t[3] * DD_ptr[3]  + t[4] * DD_ptr[21] + t[5] * DD_ptr[33]
                  + t[6] * DD_ptr[5]  + t[7] * DD_ptr[23] + t[8] * DD_ptr[35];

        Ke[1] = t[0] * DD_ptr[18] + t[1] * DD_ptr[6]  + t[2] * DD_ptr[24]
                  + t[3] * DD_ptr[21] + t[4] * DD_ptr[9]  + t[5] * DD_ptr[27]
                  + t[6] * DD_ptr[23] + t[7] * DD_ptr[11]+ t[8] * DD_ptr[29];

        Ke[2] = t[0] * DD_ptr[30] + t[1] * DD_ptr[24] + t[2] * DD_ptr[12]
                  + t[3] * DD_ptr[33] + t[4] * DD_ptr[27] + t[5] * DD_ptr[15]
                  + t[6] * DD_ptr[35] + t[7] * DD_ptr[29] + t[8] * DD_ptr[17];

        Ke[3] = t[0] * DD_ptr[3]   + t[1] * DD_ptr[21] + t[2] * DD_ptr[33]
                  + t[3] * DD_ptr[1]   + t[4] * DD_ptr[19] + t[5] * DD_ptr[31]
                  + t[6] * DD_ptr[4]   + t[7] * DD_ptr[22] + t[8] * DD_ptr[34];

        Ke[4] = t[0] * DD_ptr[21] + t[1] * DD_ptr[9]   + t[2] * DD_ptr[27]
                  + t[3] * DD_ptr[19] + t[4] * DD_ptr[7]   + t[5] * DD_ptr[25]
                  + t[6] * DD_ptr[22] + t[7] * DD_ptr[10] + t[8] * DD_ptr[28];

        Ke[5] = t[0] * DD_ptr[33] + t[1] * DD_ptr[27] + t[2] * DD_ptr[15]
                  + t[3] * DD_ptr[31] + t[4] * DD_ptr[25] + t[5] * DD_ptr[13]
                  + t[6] * DD_ptr[34] + t[7] * DD_ptr[28] + t[8] * DD_ptr[16];

        Ke[6] = t[0] * DD_ptr[5]   + t[1] * DD_ptr[23] + t[2] * DD_ptr[35]
                  + t[3] * DD_ptr[4]   + t[4] * DD_ptr[22] + t[5] * DD_ptr[34]
                  + t[6] * DD_ptr[2]   + t[7] * DD_ptr[20] + t[8] * DD_ptr[32];

        Ke[7] = t[0] * DD_ptr[23] + t[1] * DD_ptr[11] + t[2] * DD_ptr[29]
                  + t[3] * DD_ptr[22] + t[4] * DD_ptr[10] + t[5] * DD_ptr[28]
                  + t[6] * DD_ptr[20] + t[7] * DD_ptr[8]   + t[8] * DD_ptr[26];

        Ke[8] = t[0] * DD_ptr[35] + t[1] * DD_ptr[29] + t[2] * DD_ptr[17]
                  + t[3] * DD_ptr[34] + t[4] * DD_ptr[28] + t[5] * DD_ptr[16]
                  + t[6] * DD_ptr[32] + t[7] * DD_ptr[26]  + t[8] * DD_ptr[14];
        //assembly [a block of stiffness matrix]3*3
        tmpMatrix_ptr[(posn + 0) * 24 + (posm + 0)] +=  Ke[0];
        tmpMatrix_ptr[(posn + 1) * 24 + (posm + 0)] +=  Ke[1];
        tmpMatrix_ptr[(posn + 2) * 24 + (posm + 0)] +=  Ke[2];
        tmpMatrix_ptr[(posn + 0) * 24 + (posm + 1)] +=  Ke[3];
        tmpMatrix_ptr[(posn + 1) * 24 + (posm + 1)] +=  Ke[4];
        tmpMatrix_ptr[(posn + 2) * 24 + (posm + 1)] +=  Ke[5];
        tmpMatrix_ptr[(posn + 0) * 24 + (posm + 2)] +=  Ke[6];
        tmpMatrix_ptr[(posn + 1) * 24 + (posm + 2)] +=  Ke[7];
        tmpMatrix_ptr[(posn + 2) * 24 + (posm + 2)] +=  Ke[8];

        }
    }

    mdn=mdn + tmpMatrix;
    }

// qDebug() << "*********************************stiffness Matrix";
//for (int k = 0; k < 24; k++) {
//        for (int l = 0; l < 24; l++) {
//                printf("%14.6e",  mdn(k, l));
//        }
//        printf("\n");
//}
//qDebug() << "*********************************";

  MElementStiffMatrixData mData;
  mdn >> mData;
  mData.setId(data.getId());

  return mData;
  }

//        tmpMatrix_ptr[(posn + 0) * 24 + (posm + 0)] = tmpMatrix_ptr[(posn
//            + 0) * 24 + (posm + 0)] + Ke[0];
//        tmpMatrix_ptr[(posn + 1) * 24 + (posm + 0)] = tmpMatrix_ptr[(posn
//            + 1) * 24 + (posm + 0)] + Ke[1];
//        tmpMatrix_ptr[(posn + 2) * 24 + (posm + 0)] = tmpMatrix_ptr[(posn
//            + 2) * 24 + (posm + 0)] + Ke[2];
//        tmpMatrix_ptr[(posn + 0) * 24 + (posm + 1)] = tmpMatrix_ptr[(posn
//            + 0) * 24 + (posm + 1)] + Ke[3];
//        tmpMatrix_ptr[(posn + 1) * 24 + (posm + 1)] = tmpMatrix_ptr[(posn
//            + 1) * 24 + (posm + 1)] + Ke[4];
//        tmpMatrix_ptr[(posn + 2) * 24 + (posm + 1)] = tmpMatrix_ptr[(posn
//            + 2) * 24 + (posm + 1)] + Ke[5];
//        tmpMatrix_ptr[(posn + 0) * 24 + (posm + 2)] = tmpMatrix_ptr[(posn
//            + 0) * 24 + (posm + 2)] + Ke[6];
//        tmpMatrix_ptr[(posn + 1) * 24 + (posm + 2)] = tmpMatrix_ptr[(posn
//            + 1) * 24 + (posm + 2)] + Ke[7];
//        tmpMatrix_ptr[(posn + 2) * 24 + (posm + 2)] = tmpMatrix_ptr[(posn
//            + 2) * 24 + (posm + 2)] + Ke[8];
