/*
 * mchexahandlerimpl.cxx
 *
 *  Created on: 2013-6-8
 *      Author: steven
 */

#include <mchexahandlerimpl.h>
#include <qdebug.h>
#include <mextensionmanager.h>
#include <mobjectmanager.h>
#include <org.sipesc.core.utility.utilitymanager.h>
#include <org.sipesc.core.engdbs.data.mdatabasemanager.h>
#include <org.sipesc.core.engdbs.data.mdatafactorymanager.h>
#include <org.sipesc.fems.global.mfemsglobal.h>
#include <org.sipesc.fems.global.melementsglobal.h>
#include <org.sipesc.fems.data.melementdata.h>
using namespace org::sipesc::fems::global;
using namespace org::sipesc::fems::data;

class MCHEXAHandlerImpl::Data
  {
  public:
    Data()
      {
      _isInitialized = false;
      }

  public:
    MDataModel _model, _elementPath;
    MDataManager _hexManager;
    MObjectManager _objManager;
    ProgressMonitor _monitor;
    MDataFactory _eleFactory;
    bool _isInitialized;
  };

MCHEXAHandlerImpl::MCHEXAHandlerImpl()
  {
  _data.reset(new MCHEXAHandlerImpl::Data);

  if (!_data.get())
    mReportError(M_ERROR_FATAL, "MCHEXAHandlerImpl::"
        "MCHEXAHandlerImpl() ****** failed");

  _data->_objManager = MObjectManager::getManager();
  UtilityManager util = _data->_objManager.getObject(
      "org.sipesc.core.utility.utilitymanager");
  Q_ASSERT(!util.isNull());
  _data->_monitor = util.createProgressMonitor("MCHEXAHandler",
      ProgressMonitor());
  Q_ASSERT(!_data->_monitor.isNull());
  }

MCHEXAHandlerImpl::~MCHEXAHandlerImpl()
  {
  }

bool MCHEXAHandlerImpl::initialize(MDataModel& model, bool isRepeated)
  {
  if (_data->_isInitialized)
    return false;
  _data->_model = model;

  MFemsGlobal global = _data->_objManager.getObject(
      "org.sipesc.fems.global.femsglobal");
  MElementsGlobal eleGlobal = _data->_objManager.getObject(
      "org.sipesc.fems.global.elementsglobal");
  MDatabaseManager baseManager = _data->_objManager.getObject(
      "org.sipesc.core.engdbs.mdatabasemanager");
  Q_ASSERT(!baseManager.isNull());
  MDataFactoryManager factoryManager = _data->_objManager.getObject(
      "org.sipesc.core.engdbs.mdatafactorymanager");
  Q_ASSERT(!factoryManager.isNull());
  QString name = "org.sipesc.fems.data.hexabrickelementdata";
  _data->_eleFactory = factoryManager.getFactory(name);
  Q_ASSERT(!_data->_eleFactory.isNull());

  _data->_elementPath = baseManager.createDataModel();
  bool ok = _data->_elementPath.open(_data->_model,
      global.getValue(MFemsGlobal::ElementPath));
  if (!ok)
    {
    QString errorMessage = "can't open Element DataModel"
        " in MCHEXAHandlerImpl::initialize()";
    qDebug() << errorMessage;
    _data->_monitor.setMessage(errorMessage);
    return false;
    }

  _data->_hexManager = baseManager.createDataManager();
  ok = _data->_hexManager.open(_data->_elementPath,
      eleGlobal.getValue(MElementsGlobal::HexaBrickElement));
  if (!ok)
    {
    QString errorMessage = "can't open EightNodesHexElement DataManager"
        " in  MCHEXAHandlerImpl::initialize()";
    qDebug() << errorMessage;
    _data->_monitor.setMessage(errorMessage);
    return false;
    }

  _data->_isInitialized = true;
  return true;
  }

ProgressMonitor MCHEXAHandlerImpl::getProgressMonitor() const
  {
  return _data->_monitor;
  }

bool MCHEXAHandlerImpl::handleEntry(const MBdfEntryReader& reader)
  {
  Q_ASSERT(_data->_isInitialized);

  MElementData eleData = _data->_eleFactory.createObject();
  //Id
  int eleId = reader.getDataAt(0).toInt();
  //      qDebug() << "eleId = " << eleId;
  eleData.setId(eleId);
  //单元性质引用Id
  eleData.setPropertyId(reader.getDataAt(1).toInt());
  //单元节点Id
  for (int i = 0; i < 8; i++)
    {
    int nodeId = reader.getDataAt(2 + i).toInt();
    eleData.setNodeId(i, nodeId);
    }
  _data->_hexManager.appendData(eleData);

  return true;
  }
