#include <mghexabricknodestresscalculatorimpl.h>
#include <math.h>
#include <mextensionmanager.h>
#include <mobjectmanager.h>
#include <org.sipesc.core.engdbs.data.mdataobjectlist.h>
#include <org.sipesc.core.engdbs.data.mdatafactorymanager.h>
#include <org.sipesc.fems.matrix.mvectorfactory.h>
#include <org.sipesc.fems.matrix.mvectordata.h>
#include <org.sipesc.fems.matrix.mmatrixfactory.h>
#include <org.sipesc.fems.matrix.mmatrixdata.h>
#include <org.sipesc.fems.matrix.mmatrixtools.h>
using namespace org::sipesc::fems::matrix;

class MGHexaBrickNodeStressCalculatorImpl::Data
  {
  public:
    Data()
      {
      }

  public:
    MDataFactory _listFactory;
    MVectorFactory _vFactory;
    MMatrixFactory _mFactory;
    MMatrix _extrapolationMatrix;
    MMatrixTools _mTools;

  public:
    bool initialize()
      {
      MObjectManager objManager = MObjectManager::getManager();
      _mFactory = objManager.getObject(
          "org.sipesc.fems.matrix.matrixfactory");
      Q_ASSERT(!_mFactory.isNull());
      _vFactory = objManager.getObject(
          "org.sipesc.fems.matrix.vectorfactory");
      Q_ASSERT(!_vFactory.isNull());
      MDataFactoryManager factoryManager = objManager.getObject(
          "org.sipesc.core.engdbs.mdatafactorymanager");
      Q_ASSERT(!factoryManager.isNull());
      QString name = "org.sipesc.core.engdbs.MDataObjectList";
      _listFactory = factoryManager.getFactory(name);
      Q_ASSERT(!_listFactory.isNull());

      MExtensionManager extManager = MExtensionManager::getManager();
      _mTools = extManager.createExtension(
          "org.sipesc.fems.matrix.MMatrixTools");
      Q_ASSERT(!_mTools.isNull());

      MMatrix interpolation = _mFactory.createMatrix(8, 8); //应力插值矩阵与应力外推时的转换阵
      double ksi, eit, cait;
      int m = 0;
      for (int k = 0; k < 2; k++)
        for (int j = 0; j < 2; j++)
          for (int i = 0; i < 2; i++)
            {
            ksi = pow(-1, i) / sqrt(3);
            eit = pow(-1, j) / sqrt(3);
            cait = pow(-1, k) / sqrt(3);
            interpolation(m, 0, (1 - ksi) * (1 - eit) * (1 - cait) / 8);
            interpolation(m, 1, (1 + ksi) * (1 - eit) * (1 - cait) / 8);
            interpolation(m, 2, (1 + ksi) * (1 + eit) * (1 - cait) / 8);
            interpolation(m, 3, (1 - ksi) * (1 + eit) * (1 - cait) / 8);
            interpolation(m, 4, (1 - ksi) * (1 - eit) * (1 + cait) / 8);
            interpolation(m, 5, (1 + ksi) * (1 - eit) * (1 + cait) / 8);
            interpolation(m, 6, (1 + ksi) * (1 + eit) * (1 + cait) / 8);
            interpolation(m, 7, (1 - ksi) * (1 + eit) * (1 + cait) / 8);
            m++;
            }

      _extrapolationMatrix = _mTools.inverse(interpolation);
      return true;
      }
  };

MGHexaBrickNodeStressCalculatorImpl::MGHexaBrickNodeStressCalculatorImpl()
  {
  _data.reset(new MGHexaBrickNodeStressCalculatorImpl::Data);

  if (!_data.get())
    mReportError(M_ERROR_FATAL, "MGHexaBrickNodeStressCalculatorImpl::"
        "MGHexaBrickNodeStressCalculatorImpl() ****** failed");

  if (!_data->initialize()) //Data初始化
    mReportError(M_ERROR_FATAL, "MGHexaBrickNodeStressCalculatorImpl::"
        "MGHexaBrickNodeStressCalculatorImpl() ****** initialize failed");
  }

MGHexaBrickNodeStressCalculatorImpl::~MGHexaBrickNodeStressCalculatorImpl()
  {
  }

MDataObject MGHexaBrickNodeStressCalculatorImpl::getNodeStress(
    const MDataObject& integStressData)
  {
  MDataObjectList integralPoints = integStressData; //integralPoints ,nodePoints分别存放传进来的高斯点应力和坐标转换前的外推节点应力
  Q_ASSERT(!integralPoints.isNull());
  int eleId = integStressData.getId();

  MVectorData sigmaData1 = integralPoints.getDataAt(0);
  MVector sigma1 = _data->_vFactory.createVector(6);
  sigma1 << sigmaData1;
  MVectorData sigmaData2 = integralPoints.getDataAt(1);
  MVector sigma2 = _data->_vFactory.createVector(6);
  sigma2 << sigmaData2;
  MVectorData sigmaData3 = integralPoints.getDataAt(2);
  MVector sigma3 = _data->_vFactory.createVector(6);
  sigma3 << sigmaData3;
  MVectorData sigmaData4 = integralPoints.getDataAt(3);
  MVector sigma4 = _data->_vFactory.createVector(6);
  sigma4 << sigmaData4;
  MVectorData sigmaData5 = integralPoints.getDataAt(4);
  MVector sigma5 = _data->_vFactory.createVector(6);
  sigma5 << sigmaData5;
  MVectorData sigmaData6 = integralPoints.getDataAt(5);
  MVector sigma6 = _data->_vFactory.createVector(6);
  sigma6 << sigmaData6;
  MVectorData sigmaData7 = integralPoints.getDataAt(6);
  MVector sigma7 = _data->_vFactory.createVector(6);
  sigma7 << sigmaData7;
  MVectorData sigmaData8 = integralPoints.getDataAt(7);
  MVector sigma8 = _data->_vFactory.createVector(6);
  sigma8 << sigmaData8;

  MVector integralVector = _data->_vFactory.createVector(8);
  MVector tmp1 = _data->_vFactory.createVector(8);
  int vCount = sigma1.getCount();
  for (int i = 0; i < vCount; i++) //i为应力分量个数
    {
    integralVector(0, sigma1(i)); //integralVector为八个高斯点应力值的第i个分量
    integralVector(1, sigma2(i));
    integralVector(2, sigma3(i));
    integralVector(3, sigma4(i));
    integralVector(4, sigma5(i));
    integralVector(5, sigma6(i));
    integralVector(6, sigma7(i));
    integralVector(7, sigma8(i));
    tmp1 = _data->_extrapolationMatrix * integralVector; //tmp1为转化8个节点应力值的第i个分量
    sigma1(i, tmp1(0)); //把tmp1中的8个节点应力值的第i个分量分别分给八个节点应力向量
    sigma2(i, tmp1(1));
    sigma3(i, tmp1(2));
    sigma4(i, tmp1(3));
    sigma5(i, tmp1(4));
    sigma6(i, tmp1(5));
    sigma7(i, tmp1(6));
    sigma8(i, tmp1(7));
    }

  //ADDITIONAL OUTPUT CODE//
//  QList<MVector> nodePoints;
//  QList<double*> sigma_ptr_list;
//  sigma_ptr_list << sigma1.getData() << sigma2.getData() << sigma3.getData()
//      << sigma4.getData() << sigma5.getData() << sigma6.getData()
//      << sigma7.getData() << sigma8.getData();
//  for (int i = 0; i < 8; i++)
//    {
//
//    double* s = sigma_ptr_list[i];
//    double tmpmises = (s[0] - s[1]) * (s[0] - s[1])
//        + (s[1] - s[2]) * (s[1] - s[2]) + (s[2] - s[0]) * (s[2] - s[0])
//        + 6 * (s[3] * s[3] + s[4] * s[4] + s[5] * s[5]);
//    tmpmises = sqrt(tmpmises / 2);
//    double tmppres = (s[0] + s[1] + s[2]) / 3;
//
//    MVector ampsigma = _data->_vFactory.createVector(8);
//    double* ampsigma_ptr = ampsigma.getData();
//
//    ampsigma_ptr[0] = s[0];
//    ampsigma_ptr[1] = s[1];
//    ampsigma_ptr[2] = s[2];
//    ampsigma_ptr[3] = s[3];
//    ampsigma_ptr[4] = s[4];
//    ampsigma_ptr[5] = s[5];
//    ampsigma_ptr[6] = tmpmises;
//    ampsigma_ptr[7] = tmppres;
//
//    nodePoints << ampsigma;
//    }

  MDataObjectList nodeStressVectors = _data->_listFactory.createObject(); //nodeStressVectors存放坐标转换后的节点应力
  nodeStressVectors.setId(eleId);
//  for (int i = 0; i < 8; i++)
//    {
//    MVector tmp = nodePoints[i];
//    MVectorData nodeVectorData;
//    tmp >> nodeVectorData;
//    nodeStressVectors.appendData(nodeVectorData);
//    }

    sigma1 >> sigmaData1;
    sigma2 >> sigmaData2;
    sigma3 >> sigmaData3;
    sigma4 >> sigmaData4;
    sigma5 >> sigmaData5;
    sigma6 >> sigmaData6;
    sigma7 >> sigmaData7;
    sigma8 >> sigmaData8;
    nodeStressVectors.appendData(sigmaData1); //nodeStressVectors是坐标变换前的节点应力数组
    nodeStressVectors.appendData(sigmaData2); //而nodeStressVectors是坐标转换后的
    nodeStressVectors.appendData(sigmaData3); //总体坐标系下的节点应力数组
    nodeStressVectors.appendData(sigmaData4);
    nodeStressVectors.appendData(sigmaData5);
    nodeStressVectors.appendData(sigmaData6);
    nodeStressVectors.appendData(sigmaData7);
    nodeStressVectors.appendData(sigmaData8);

  return nodeStressVectors;
  }
