#include <mhexabricklumpedmassmatriximpl.h>
#include <qdebug.h>
#include <mextensionmanager.h>
#include <mobjectmanager.h>
#include <org.sipesc.core.utility.utilitymanager.h>
#include <org.sipesc.utilities.mextensionfactorymanager.h>
#include <org.sipesc.fems.matrix.mmatrixfactory.h>
#include <org.sipesc.fems.data.melementdata.h>
#include <org.sipesc.fems.hexaelement.mhexabrickelevolume.h>
using namespace org::sipesc::utilities;
using namespace org::sipesc::fems::matrix;
using namespace org::sipesc::fems::data;

class MHexaBrickLumpedMassMatrixImpl::Data
  {
  public:
    Data()
      {
      _isInitialized = false;
      }

  public:
    MDataModel _model;
    MObjectManager _objManager;
    ProgressMonitor _monitor;
    MMatrix _lumpedMass;
    MHexaBrickEleVolume _volParser;
    bool _isInitialized;
  };

MHexaBrickLumpedMassMatrixImpl::MHexaBrickLumpedMassMatrixImpl()
  {
  _data.reset(new MHexaBrickLumpedMassMatrixImpl::Data);

  if (!_data.get())
    mReportError(M_ERROR_FATAL, "MHexaBrickLumpedMassMatrixImpl::"
        "MHexaBrickLumpedMassMatrixImpl() ****** failed");

  _data->_objManager = MObjectManager::getManager();
  UtilityManager util = _data->_objManager.getObject(
      "org.sipesc.core.utility.utilitymanager");
  Q_ASSERT(!util.isNull());
  _data->_monitor = util.createProgressMonitor("MHexaBrickLumpedMassMatrix",
      ProgressMonitor());
  Q_ASSERT(!_data->_monitor.isNull());
  }

MHexaBrickLumpedMassMatrixImpl::~MHexaBrickLumpedMassMatrixImpl()
  {
  }

bool MHexaBrickLumpedMassMatrixImpl::initialize(MDataModel& model,
    bool isRepeated)
  {
  if (_data->_isInitialized)
    return false; //不能重复初始化

  _data->_model = model;
  MMatrixFactory mFactory = _data->_objManager.getObject(
      "org.sipesc.fems.matrix.matrixfactory");
  Q_ASSERT(!mFactory.isNull());

  //创建六面体体积解析扩展
  MExtensionManager extManager = MExtensionManager::getManager();
  _data->_volParser = extManager.createExtension(
      "org.sipesc.fems.hexaelement.MHexaBrickEleVolume");
  Q_ASSERT(!_data->_volParser.isNull());
  bool isOk = _data->_volParser.initialize(_data->_model);
  if (!isOk)
    {
    QString errorMessage = "can't initialize MHexaBrickEleVolume"
        " in MHexaBrickLumpedMassMatrixImpl::initialize() ";
    qDebug() << errorMessage;
    _data->_monitor.setMessage(errorMessage);
    return false;
    }

  MMatrix mdn = mFactory.createMatrix(24, 1);
  for (int i = 0; i < 24; i++)
    mdn(i, 0, 1.0 / 8.0);

  _data->_lumpedMass = mdn;

  _data->_isInitialized = true;
  return true;
  }

ProgressMonitor MHexaBrickLumpedMassMatrixImpl::getProgressMonitor() const
  {
  return _data->_monitor;
  }

void MHexaBrickLumpedMassMatrixImpl::setState(const MDataObject& eleData)
  {
  Q_ASSERT(_data->_isInitialized);
  MElementData data = eleData;
  Q_ASSERT(!data.isNull());

  _data->_volParser.setState(data);
  }

MMatrix MHexaBrickLumpedMassMatrixImpl::getMassMatrix(
    const MElementMass& eleMass) const
  {
  double density = eleMass.getEleMass();
  double volume = _data->_volParser.getEleVolume();

  return _data->_lumpedMass * density * volume;
  }
