#include <mhexabricknodestresscalculatorimpl.h>
#include <math.h>
#include <mextensionmanager.h>
#include <mobjectmanager.h>
#include <org.sipesc.core.engdbs.data.mdataobjectlist.h>
#include <org.sipesc.core.engdbs.data.mdatafactorymanager.h>
#include <org.sipesc.fems.matrix.mvectorfactory.h>
#include <org.sipesc.fems.matrix.mvectordata.h>
#include <org.sipesc.fems.matrix.mmatrixfactory.h>
#include <org.sipesc.fems.matrix.mmatrixdata.h>
#include <org.sipesc.fems.matrix.mmatrixtools.h>
#include <org.sipesc.fems.data.melementdata.h>
#include <org.sipesc.fems.data.melementlocalcoorddata.h>
using namespace org::sipesc::fems::matrix;
using namespace org::sipesc::fems::data;

class MHexaBrickNodeStressCalculatorImpl::Data
  {
  public:
    Data()
      {
      }

  public:
    MDataFactory _listFactory;
    MVectorFactory _vFactory;
    MMatrixFactory _mFactory;
    MMatrix _extrapolationMatrix;
    MMatrixTools _mTools;
    MElementLocalCoordData _coordData;

  public:
    bool initialize()
      {
      MObjectManager objManager = MObjectManager::getManager();
      _mFactory = objManager.getObject(
          "org.sipesc.fems.matrix.matrixfactory");
      Q_ASSERT(!_mFactory.isNull());
      _vFactory = objManager.getObject(
          "org.sipesc.fems.matrix.vectorfactory");
      Q_ASSERT(!_vFactory.isNull());
      MDataFactoryManager factoryManager = objManager.getObject(
          "org.sipesc.core.engdbs.mdatafactorymanager");
      Q_ASSERT(!factoryManager.isNull());
      QString name = "org.sipesc.core.engdbs.MDataObjectList";
      _listFactory = factoryManager.getFactory(name);
      Q_ASSERT(!_listFactory.isNull());

      MExtensionManager extManager = MExtensionManager::getManager();
      _mTools = extManager.createExtension(
          "org.sipesc.fems.matrix.MMatrixTools");
      Q_ASSERT(!_mTools.isNull());

      MMatrix interpolation = _mFactory.createMatrix(8, 8); //应力插值矩阵与应力外推时的转换阵
      double ksi, eit, cait;
      int m = 0;
      for (int k = 0; k < 2; k++)
        for (int j = 0; j < 2; j++)
          for (int i = 0; i < 2; i++)
            {
            ksi = pow(-1, i) / sqrt(3);
            eit = pow(-1, j) / sqrt(3);
            cait = pow(-1, k) / sqrt(3);
            interpolation(m, 0, (1 - ksi) * (1 - eit) * (1 - cait) / 8);
            interpolation(m, 1, (1 + ksi) * (1 - eit) * (1 - cait) / 8);
            interpolation(m, 2, (1 + ksi) * (1 + eit) * (1 - cait) / 8);
            interpolation(m, 3, (1 - ksi) * (1 + eit) * (1 - cait) / 8);
            interpolation(m, 4, (1 - ksi) * (1 - eit) * (1 + cait) / 8);
            interpolation(m, 5, (1 + ksi) * (1 - eit) * (1 + cait) / 8);
            interpolation(m, 6, (1 + ksi) * (1 + eit) * (1 + cait) / 8);
            interpolation(m, 7, (1 - ksi) * (1 + eit) * (1 + cait) / 8);
            m++;
            }

      _extrapolationMatrix = _mTools.inverse(interpolation);
      return true;
      }
  };

MHexaBrickNodeStressCalculatorImpl::MHexaBrickNodeStressCalculatorImpl()
  {
  _data.reset(new MHexaBrickNodeStressCalculatorImpl::Data);

  if (!_data.get())
    mReportError(M_ERROR_FATAL, "MHexaBrickNodeStressCalculatorImpl::"
        "MHexaBrickNodeStressCalculatorImpl() ****** failed");

  if (!_data->initialize()) //Data初始化
    mReportError(M_ERROR_FATAL, "MHexaBrickNodeStressCalculatorImpl::"
        "MHexaBrickNodeStressCalculatorImpl() ****** initialize failed");
  }

MHexaBrickNodeStressCalculatorImpl::~MHexaBrickNodeStressCalculatorImpl()
  {
  }

void MHexaBrickNodeStressCalculatorImpl::setState(
    const MDataObject& localData)
  {
  _data->_coordData = localData;
  Q_ASSERT(!_data->_coordData.isNull());
  }

MDataObject MHexaBrickNodeStressCalculatorImpl::getNodeStress(
    const MDataObject& integStressData)
  {
  MDataObjectList integralPoints = integStressData; //integralPoints ,nodePoints分别存放传进来的高斯点应力和坐标转换前的外推节点应力
  Q_ASSERT(!integralPoints.isNull());
  int eleId = integStressData.getId();

  MVectorData sigmaData1 = integralPoints.getDataAt(0);
  MVector sigma1 = _data->_vFactory.createVector(6);
  sigma1 << sigmaData1;
  MVectorData sigmaData2 = integralPoints.getDataAt(1);
  MVector sigma2 = _data->_vFactory.createVector(6);
  sigma2 << sigmaData2;
  MVectorData sigmaData3 = integralPoints.getDataAt(2);
  MVector sigma3 = _data->_vFactory.createVector(6);
  sigma3 << sigmaData3;
  MVectorData sigmaData4 = integralPoints.getDataAt(3);
  MVector sigma4 = _data->_vFactory.createVector(6);
  sigma4 << sigmaData4;
  MVectorData sigmaData5 = integralPoints.getDataAt(4);
  MVector sigma5 = _data->_vFactory.createVector(6);
  sigma5 << sigmaData5;
  MVectorData sigmaData6 = integralPoints.getDataAt(5);
  MVector sigma6 = _data->_vFactory.createVector(6);
  sigma6 << sigmaData6;
  MVectorData sigmaData7 = integralPoints.getDataAt(6);
  MVector sigma7 = _data->_vFactory.createVector(6);
  sigma7 << sigmaData7;
  MVectorData sigmaData8 = integralPoints.getDataAt(7);
  MVector sigma8 = _data->_vFactory.createVector(6);
  sigma8 << sigmaData8;

  MVector integralVector = _data->_vFactory.createVector(8);
  MVector tmp1 = _data->_vFactory.createVector(8);
  for (int i = 0; i < 6; i++) //i为应力分量个数
    {
    integralVector(0, sigma1(i)); //integralVector为八个高斯点应力值的第i个分量
    integralVector(1, sigma2(i));
    integralVector(2, sigma3(i));
    integralVector(3, sigma4(i));
    integralVector(4, sigma5(i));
    integralVector(5, sigma6(i));
    integralVector(6, sigma7(i));
    integralVector(7, sigma8(i));
    tmp1 = _data->_extrapolationMatrix * integralVector; //tmp1为转化8个节点应力值的第i个分量
    sigma1(i, tmp1(0)); //把tmp1中的8个节点应力值的第i个分量分别分给八个节点应力向量
    sigma2(i, tmp1(1));
    sigma3(i, tmp1(2));
    sigma4(i, tmp1(3));
    sigma5(i, tmp1(4));
    sigma6(i, tmp1(5));
    sigma7(i, tmp1(6));
    sigma8(i, tmp1(7));
    }

  QList<MVector> nodePoints;
  nodePoints << sigma1 << sigma2 << sigma3 << sigma4 << sigma5 << sigma6
      << sigma7 << sigma8;

//  sigma1 >> sigmaData1;
//  sigma2 >> sigmaData2;
//  sigma3 >> sigmaData3;
//  sigma4 >> sigmaData4;
//  sigma5 >> sigmaData5;
//  sigma6 >> sigmaData6;
//  sigma7 >> sigmaData7;
//  sigma8 >> sigmaData8;
//
//  MDataObjectList nodePoints = _data->_listFactory.createObject();
//  nodePoints.appendData(sigmaData1); //nodePoints是坐标变换前的节点应力数组
//  nodePoints.appendData(sigmaData2); //而nodeStressVectors是坐标转换后的
//  nodePoints.appendData(sigmaData3); //总体坐标系下的节点应力数组
//  nodePoints.appendData(sigmaData4);
//  nodePoints.appendData(sigmaData5);
//  nodePoints.appendData(sigmaData6);
//  nodePoints.appendData(sigmaData7);
//  nodePoints.appendData(sigmaData8);

  MMatrixData mData = _data->_coordData.getTransMatrixData();
  MMatrix m = _data->_mFactory.createMatrix();
  m << mData;

  MMatrix trans = _data->_mFactory.createMatrix(6, 6);
  trans(0, 0, m(0, 0) * m(0, 0));
  trans(0, 1, m(0, 1) * m(0, 1));
  trans(0, 2, m(0, 2) * m(0, 2));
  trans(1, 0, m(1, 0) * m(1, 0));
  trans(1, 1, m(1, 1) * m(1, 1));
  trans(1, 2, m(1, 2) * m(1, 2));
  trans(2, 0, m(2, 0) * m(2, 0));
  trans(2, 1, m(2, 1) * m(2, 1));
  trans(2, 2, m(2, 2) * m(2, 2));

  trans(0, 3, m(0, 0) * m(0, 1));
  trans(0, 4, m(0, 1) * m(0, 2));
  trans(0, 5, m(0, 2) * m(0, 0));
  trans(1, 3, m(1, 0) * m(1, 1));
  trans(1, 4, m(1, 1) * m(1, 2));
  trans(1, 5, m(1, 2) * m(1, 0));
  trans(2, 3, m(2, 0) * m(2, 1));
  trans(2, 4, m(2, 1) * m(2, 2));
  trans(2, 5, m(2, 2) * m(2, 0));

  trans(3, 0, 2.0 * m(0, 0) * m(1, 0));
  trans(3, 1, 2.0 * m(0, 1) * m(1, 1));
  trans(3, 2, 2.0 * m(0, 2) * m(1, 2));
  trans(4, 0, 2.0 * m(1, 0) * m(2, 0));
  trans(4, 1, 2.0 * m(1, 1) * m(2, 1));
  trans(4, 2, 2.0 * m(1, 2) * m(2, 2));
  trans(5, 0, 2.0 * m(2, 0) * m(0, 0));
  trans(5, 1, 2.0 * m(2, 1) * m(0, 1));
  trans(5, 2, 2.0 * m(2, 2) * m(0, 2));

  trans(3, 3, m(0, 0) * m(1, 1) + m(1, 0) * m(0, 1));
  trans(3, 4, m(0, 1) * m(1, 2) + m(1, 1) * m(0, 2));
  trans(3, 5, m(0, 2) * m(1, 0) + m(1, 2) * m(0, 0));
  trans(4, 3, m(1, 0) * m(2, 1) + m(2, 0) * m(1, 1));
  trans(4, 4, m(1, 1) * m(2, 2) + m(2, 1) * m(1, 2));
  trans(4, 5, m(1, 2) * m(2, 0) + m(2, 2) * m(1, 0));
  trans(5, 3, m(2, 0) * m(0, 1) + m(0, 0) * m(2, 1));
  trans(5, 4, m(2, 1) * m(0, 2) + m(0, 1) * m(2, 2));
  trans(5, 5, m(2, 2) * m(0, 0) + m(0, 2) * m(2, 0));

  MDataObjectList nodeStressVectors = _data->_listFactory.createObject(); //nodeStressVectors存放坐标转换后的节点应力
  nodeStressVectors.setId(eleId);
  for (int i = 0; i < 8; i++)
    {
//    MVectorData tmpData = nodePoints.getData(i);
//    MVector tmp = _data->_vFactory.createVector();
//    tmp << tmpData;
    MVector tmp = nodePoints[i];
    MVector nodeVector = _data->_mTools.transpose(trans) * tmp;
    MVectorData nodeVectorData;
    nodeVector >> nodeVectorData;
    nodeStressVectors.appendData(nodeVectorData);
    }

  return nodeStressVectors;
  }
