#include <mhexabrickstrainmatriximpl.h>
#include <math.h>
#include <qdebug.h>
#include <mextensionmanager.h>
#include <mobjectmanager.h>
#include <org.sipesc.fems.global.melementsglobal.h>
#include <org.sipesc.utilities.mextensionfactorymanager.h>
#include <org.sipesc.fems.matrix.mmatrixtools.h>
#include <org.sipesc.fems.data.mnodedata.h>
#include <org.sipesc.fems.data.melementlocalcoorddata.h>
#include <org.sipesc.fems.element.mshpfunction.h>
using namespace org::sipesc::fems::global;
using namespace org::sipesc::utilities;
using namespace org::sipesc::fems::data;
using namespace org::sipesc::fems::element;

class MHexaBrickStrainMatrixImpl::Data
  {
  public:
    Data() //Data的构造函数
      {
      _jacobiValue = 0.0; //对_jacobiValue初始化，对_shpFunction的初始化在initialize()中
      }

  public:
    ProgressMonitor _monitor;
    MMatrixFactory _mFactory;
    MMatrixTools _mTools;
    MShpFunction _shpFunction;
    MElementLocalCoordData _coordData;
    MMatrix _nodeCoord;
    double _jacobiValue;
    bool _isInitialized;

  public:
    bool initialize() //对_shpFunction的初始化
      {
      MObjectManager objManager = MObjectManager::getManager();
      _mFactory = objManager.getObject(
          "org.sipesc.fems.matrix.matrixfactory");
      Q_ASSERT(!_mFactory.isNull());

      //通过插件找工厂管理器
      MExtensionManager extManager = MExtensionManager::getManager();
      _shpFunction = extManager.createExtension(
          "org.sipesc.fems.hexaelement.MHexaShpFunction");
      Q_ASSERT(!_shpFunction.isNull());

      _mTools = extManager.createExtension(
          "org.sipesc.fems.matrix.MMatrixTools");
      Q_ASSERT(!_mTools.isNull());

      return true;
      }
  };

MHexaBrickStrainMatrixImpl::MHexaBrickStrainMatrixImpl()
  {
  _data.reset(new MHexaBrickStrainMatrixImpl::Data);

  if (!_data.get())
    mReportError(M_ERROR_FATAL, "MHexaBrickStrainMatrixImpl::"
        "MHexaBrickStrainMatrixImpl() ****** failed");

  if (!_data->initialize()) //Data初始化
    mReportError(M_ERROR_FATAL, "MHexaBrickStrainMatrixImpl::"
        "MHexaBrickStrainMatrixImpl() ****** initialize failed");
  }

MHexaBrickStrainMatrixImpl::~MHexaBrickStrainMatrixImpl()
  {
  }

bool MHexaBrickStrainMatrixImpl::initialize(MDataModel& model,
    bool isRepeated)
  {
  if (_data->_isInitialized)
    return false; //不能重复初始化


  _data->_isInitialized = true;
  return true;
  }

ProgressMonitor MHexaBrickStrainMatrixImpl::getProgressMonitor() const
  {
  return _data->_monitor;
  }

void MHexaBrickStrainMatrixImpl::setState(const MDataObject& localData)
  {
  _data->_coordData = localData;
  Q_ASSERT(!_data->_coordData.isNull());

  MNodeData nodeData;
  MMatrix node = _data->_mFactory.createMatrix(8, 3);
  for (int i = 0; i < 8; i++)
    {
    nodeData = _data->_coordData.getNodes().value(i);
    Q_ASSERT(!nodeData.isNull());
    node(i,  0,nodeData.getX());
    node(i, 1, nodeData.getY());
    node( i, 2,nodeData.getZ());
    //    qDebug()<<node(i,  0)<<node(i,  1)<<node(i,  2);
    }

  _data->_nodeCoord = node;
  }

MMatrix MHexaBrickStrainMatrixImpl::getStrainMatrix(
    const MIntegPoint& integ)
  {
  MMatrix shpFuncDerivative = _data->_mFactory.createMatrix(3, 8);
  shpFuncDerivative = _data->_shpFunction.getShpFuncDerivative(
      integ.getIntegCoord()); //形函数导数矩阵（在积分点的值）
  MMatrix jacobiMatrix = _data->_mFactory.createMatrix(3, 3);
  jacobiMatrix = shpFuncDerivative * _data->_nodeCoord;
  _data->_jacobiValue = _data->_mTools.determinate(jacobiMatrix); // 计算雅克比矩阵行列式值
  MMatrix invJacobiMatrix = _data->_mFactory.createMatrix(3, 3);
  invJacobiMatrix = _data->_mTools.inverse(jacobiMatrix); //雅克比矩阵的逆

  //new method
  MMatrix B = _data->_mFactory.createMatrix(3, 8);
  B=invJacobiMatrix *shpFuncDerivative;


  ////old
//    MMatrix B = _data->_mFactory.createMatrix(6, 24);
//    MMatrix BI = _data->_mFactory.createMatrix(3, 1);
//    MMatrix tmp = _data->_mFactory.createMatrix(3, 1);
//    for (int i = 0; i < 8; i++)
//      {
//      BI(0, 0, shpFuncDerivative(0, i));
//      BI(1, 0, shpFuncDerivative(1, i));
//      BI(2, 0, shpFuncDerivative(2, i));
//
//      tmp = invJacobiMatrix * BI;
//      B(0, i, tmp(0, 0));
//      B(1, i , tmp(1, 0));
//      B(2, i , tmp(2, 0));
//      B(0, i * 3, tmp(0, 0));
//      B(0, i * 3 + 1, 0.0);
//      B(0, i * 3 + 2, 0.0);
//      B(1, i * 3, 0.0);
//      B(1, i * 3 + 1, tmp(1, 0));
//      B(1, i * 3 + 2, 0.0);
//      B(2, i * 3, 0.0);
//      B(2, i * 3 + 1, 0.0);
//      B(2, i * 3 + 2, tmp(2, 0));
//
//      B(3, i * 3, tmp(1, 0));
//      B(3, i * 3 + 1, tmp(0, 0));
//      B(3, i * 3 + 2, 0.0);
//      B(4, i * 3, 0.0);
//      B(4, i * 3 + 1, tmp(2, 0));
//      B(4, i * 3 + 2, tmp(1, 0));
//      B(5, i * 3, tmp(2, 0));
//      B(5, i * 3 + 1, 0.0);
//      B(5, i * 3 + 2, tmp(0, 0));
//      }
  //
  //  qDebug() << "*********************************DNDx";
  // for (int k = 0; k < 3; k++) {
  //         for (int l = 0; l < 8; l++) {
  //                 printf("%14.6e",  B(k, l));
  //         }
  //         printf("\n");
  // }
  // qDebug() << "*********************************";
  return B;
  }




double MHexaBrickStrainMatrixImpl::getJacobiValue() const
  {
  return _data->_jacobiValue;
  }

QList<double> MHexaBrickStrainMatrixImpl::coordTrans(
    const MIntegPoint& integ) const
  {
  MMatrix shpFunc = _data->_mFactory.createMatrix(1, 8);
  shpFunc = _data->_shpFunction.getShpFunction(integ.getIntegCoord()); //得到形函数，（对应于积分点）
  MMatrix co = _data->_mFactory.createMatrix(1, 3);
  co = shpFunc * _data->_nodeCoord; //得到真实的点坐标

  MMatrixData mData = _data->_coordData.getTransMatrixData();
  MMatrix m = _data->_mFactory.createMatrix();
  m << mData; //坐标转换阵

  MMatrix coord = _data->_mFactory.createMatrix(3, 1);
  coord(0, 0, co(0, 0));
  coord(1, 0, co(0, 1));
  coord(2, 0, co(0, 2));
  MMatrix tmp = _data->_mTools.transpose(m) * coord;

  QList<double> tmpCoord;
  tmpCoord.append(tmp(0, 0));
  tmpCoord.append(tmp(1, 0));
  tmpCoord.append(tmp(2, 0));

  return tmpCoord;
  }
