#include <mhexashpfunctionimpl.h>
#include <mobjectmanager.h>
#include <qdebug.h>
MHexaShpFunctionImpl::MHexaShpFunctionImpl()
  {
  MObjectManager objManager = MObjectManager::getManager();
  _mFactory = objManager.getObject("org.sipesc.fems.matrix.matrixfactory");
  Q_ASSERT(!_mFactory.isNull());
  }

MHexaShpFunctionImpl::~MHexaShpFunctionImpl()
  {
  }

MMatrix MHexaShpFunctionImpl::getShpFunction(
    const QList<double>& paraCoord,
    const QVector<MNodeData>& nodeLocalCoord) const //以下这两个程序可以更方便
  {
  MMatrix mdn = _mFactory.createMatrix(1, 8);
  double ksi, eit, cait;
  ksi = paraCoord.value(0);
  eit = paraCoord.value(1);
  cait = paraCoord.value(2);

  mdn(0, 0, (1 - ksi) * (1 - eit) * (1 - cait) / 8);
  mdn(0, 1, (1 + ksi) * (1 - eit) * (1 - cait) / 8);
  mdn(0, 2, (1 + ksi) * (1 + eit) * (1 - cait) / 8);
  mdn(0, 3, (1 - ksi) * (1 + eit) * (1 - cait) / 8);

  mdn(0, 4, (1 - ksi) * (1 - eit) * (1 + cait) / 8);
  mdn(0, 5, (1 + ksi) * (1 - eit) * (1 + cait) / 8);
  mdn(0, 6, (1 + ksi) * (1 + eit) * (1 + cait) / 8);
  mdn(0, 7, (1 - ksi) * (1 + eit) * (1 + cait) / 8);

  return mdn;
  }

MMatrix MHexaShpFunctionImpl::getShpFuncDerivative(
    const QList<double>& paraCoord,
    const QVector<MNodeData>& nodeLocalCoord) const
  {
  MMatrix msfd= _mFactory.createMatrix(3, 8);
  double ksi, eit, cait;
  ksi = paraCoord.value(0);
  eit = paraCoord.value(1);
  cait = paraCoord.value(2);
//  qDebug()<<ksi<<eit<<cait;
  msfd(0, 0, -(1 - eit) * (1 - cait) / 8);
  msfd(1, 0, -(1 - ksi) * (1 - cait) / 8);
  msfd(2, 0, -(1 - eit) * (1 - ksi) / 8);

  msfd(0, 1, (1 - eit) * (1 - cait) / 8);
  msfd(1, 1, -(1 + ksi) * (1 - cait) / 8);
  msfd(2, 1, -(1 - eit) * (1 + ksi) / 8);

  msfd(0, 2, (1 + eit) * (1 - cait) / 8);
  msfd(1, 2, (1 + ksi) * (1 - cait) / 8);
  msfd(2, 2, -(1 + eit) * (1 + ksi) / 8);

  msfd(0, 3, -(1 + eit) * (1 - cait) / 8);
  msfd(1, 3, (1 - ksi) * (1 - cait) / 8);
  msfd(2, 3, -(1 + eit) * (1 - ksi) / 8);

  msfd(0, 4, -(1 - eit) * (1 + cait) / 8);
  msfd(1, 4, -(1 - ksi) * (1 + cait) / 8);
  msfd(2, 4, (1 - eit) * (1 - ksi) / 8);

  msfd(0, 5, (1 - eit) * (1 + cait) / 8);
  msfd(1, 5, -(1 + ksi) * (1 + cait) / 8);
  msfd(2, 5, (1 - eit) * (1 + ksi) / 8);

  msfd(0, 6, (1 + eit) * (1 + cait) / 8);
  msfd(1, 6, (1 + ksi) * (1 + cait) / 8);
  msfd(2, 6, (1 + eit) * (1 + ksi) / 8);

  msfd(0, 7, -(1 + eit) * (1 + cait) / 8);
  msfd(1, 7, (1 - ksi) * (1 + cait) / 8);
  msfd(2, 7, (1 + eit) * (1 - ksi) / 8);

  return msfd;
  }
